package com.mom.jcrperf;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.mom.jcrperf.tests.PerformanceTest;

abstract public class AbstractTestRunner {
	
	protected boolean listTests = false;
	protected boolean showConfiguration = false;
	protected boolean outputXml = false;
	
	protected PerformanceTestSuite suite = new PerformanceTestSuite();
	
	List<PerformanceTest> selectedTests = new ArrayList<PerformanceTest>();
	
	protected boolean showUsage = false;
	protected int optionWidth = 0;
	
	
	abstract public ITestableRepository getTestableRepository();
	
	public class ArgumentException extends Exception {
		
		public ArgumentException(String message) {
			super(message);
		}
	}
	

	public void showUsage() {
		optionWidth = 0;
		showUsage = false;
		usage();
		showUsage = true;
		usage();
	}
	public void usage() {
		usage("Usage: perfTest [options] -t [<test1,test2...> | *] - Run Tests");
		usage("Options:");
		usage("-h",  "Print this help");
		usage("-l",  "Lists Tests");
		usage("-c",  "Show Configuration");
		usage("-x",  "Set XML output format");
	}
	
	public void usage(String unformatted) {
		if(showUsage) {
			System.out.println(unformatted);
		}
	}
	
	public void usage(String option, String description) {
		if(showUsage) {
			System.out.printf("%-"+optionWidth+"s - %s\n", option, description);
		}
		
		if(option.length() > optionWidth) {
			optionWidth = option.length();
		}
	}
	
	public void run(String[] argv) throws Exception {
		try {
			if(parseArguments(argv)) {			
				runSuite();
			}
			
		} catch(ArgumentException e) {
			System.out.println("Error: "+e.getMessage());
			showUsage();
			return;
		} catch(Exception r) {
			System.out.println("Error: "+r.getMessage());
			try {
				getTestableRepository().shutdown();
			} catch(Exception ignored) { }
			throw r;
		}
	}
	
	public boolean  parseArguments(String[] argv) throws ArgumentException {
		// Copy the args into a list for ease of extraction
		List<String> args = new ArrayList<String>();
		args.addAll(Arrays.asList(argv));
		return extractArguments(args);
	}
	
	public boolean extractArguments(List<String> args) throws ArgumentException {
		
		// Scan args for the base set of arguments and remove any found
		int i=0;
		while(i < args.size()) {
			String arg = args.get(i);
			
			if("-h".equals(arg)) {
				showUsage();
				return false;
			}
			
			if("-l".equals(arg)) {
				// List the available tests and exit
				listTests = true;
				args.remove(i);
				continue;
			}
			
			if("-c".equals(arg)) {
				// Dump the configuration and exit. May be used in conjunction with -x
				showConfiguration = true;
				args.remove(i);
				continue;
			}
			
			if("-x".equals(arg)) {
				// Emit all output as XML
				outputXml = true;
				args.remove(i);
				continue;
			}
			
			if("-t".equals(arg)) {
				// List of tests to run
				// * is allowed for all
				if(i >= args.size()) {
					throw new ArgumentException("A (comma separated) list of tests must be supplied with the -t argument");
				}

				args.remove(i);
				arg = args.get(i);
				args.remove(i);
				
				Map<String, PerformanceTest> tests = suite.getTestsList();
				
				if("*".equals(arg)) {
					selectedTests.addAll(tests.values());
				} else {
					for(String testName : arg.split(",")) {
						if (tests.containsKey(testName)) {
							selectedTests.add(tests.get(testName));
						} else {
							throw new ArgumentException("Unknown test: "+testName);
						}
					}
				}
				continue;
			}
			
			i++;
		}
		
		return true;
	}
	
	public void runSuite() throws Exception {

		suite = new PerformanceTestSuite();
		
		if(listTests) {
			listTests();
			return;
		}
		
		ITestableRepository testable = getTestableRepository();
		testable.initialise();
		suite.setTestable(testable);

		
		if(outputXml) {
			System.out.println("<?xml version=\"1.0\"?>");
			System.out.println("<testSuite>");
		}
		
		if (showConfiguration) {
			
			showConfig(testable.getConfiguration());
			showConfig(getSystemConfiguration());
		}
		
		if(!outputXml) {
			System.out.println("Test case                           " + "    Mean    Sdev     Min     Max   Count");
			System.out.println("------------------------------------" + "----------------------------------------");
		}
		
		runTests();
		
		if(outputXml) {
			System.out.println("</testSuite>");
		}
		
		testable.shutdown();
	}
	

	public void listTests() {
		Map<String, PerformanceTest> tests = suite.getTestsList();
		System.out.println("Available tests");
		System.out.println("---------------------------------------------");
		for (String test : tests.keySet()) {
			System.out.println(test);
		}
	}
	
	public void showConfig(Map<String, Map<String, String>> config) {
		if(outputXml) {
			for(String section : config.keySet()) {
				System.out.println("<"+section+">");
				for(String property : config.get(section).keySet()) {
					System.out.println("<"+property+">"+config.get(section).get(property)+"</"+property+">");
				}
				System.out.println("</"+section+">");
			}
		} else {
			for(String section : config.keySet()) {
				for(String property : config.get(section).keySet()) {
					System.out.println(property+" = "+config.get(section).get(property));
				}
			}
		}
	}
	
	public Map<String, Map<String, String>> getSystemConfiguration() {
		
		Map<String, Map<String, String>> config = new HashMap<String, Map<String,String>>();
		
		Map<String, String> jre = new HashMap<String, String>();
		jre.put("version", System.getProperty("java.version"));
		jre.put("vendor", System.getProperty("java.version"));
		config.put("jre", jre);
		
		Map<String, String> jvm = new HashMap<String, String>();
		jvm.put("name", System.getProperty("java.vm.name"));
		jvm.put("version", System.getProperty("java.vm.version"));
		jvm.put("vendor", System.getProperty("java.vm.vendor"));
		config.put("jvm", jvm);
		
		Map<String, String> os = new HashMap<String, String>();
		os.put("name", System.getProperty("os.name"));
		os.put("version", System.getProperty("os.version"));
		os.put("arch", System.getProperty("os.arch"));
		config.put("os", os);
				
		return config;
	}
	
	public void runTests() throws Exception {
		
		// Run the tests
		for (PerformanceTest test : selectedTests) {
			suite.run(test, outputXml);
		}		
	}

}
