package com.mom.jcrperf;

import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

import javax.jcr.Credentials;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.sql.DataSource;

/**
 * This interface defines a wrapper class which is used to provide
 * a fully-configured implementation of a JCR repository for testing
 * 
 * @author duncan
 *
 */

public abstract class AbstractTestableRepository implements ITestableRepository {
	
	protected Repository repository;
	protected Credentials credentials;
	
	public abstract void initialise() throws Exception;
	public abstract void shutdown();
	
	

	
	public Repository getRepository() {
		return repository;
	}
	
	public Credentials getCredentials() {
		return credentials;
	}

	// Deletes all files and subdirectories under dir.
	// Returns true if all deletions were successful.
	// If a deletion fails, the method stops attempting to delete and returns false.
	public static boolean deleteDir(File dir) {
	    if (dir.isDirectory()) {
	        String[] children = dir.list();
	        for (int i=0; i<children.length; i++) {
	            boolean success = deleteDir(new File(dir, children[i]));
	            if (!success) {
	                return false;
	            }
	        }
	    }

	    // The directory is now empty so delete it
	    return dir.delete();
	}
	

    /**
     * Open a connection using the specified properties.
     * The connection can be created using a JNDI Data Source as well. To do that,
     * the driver class name must reference a javax.naming.Context class
     * (for example javax.naming.InitialContext), and the URL must be the JNDI URL
     * (for example java:comp/env/jdbc/Test).
     *
     * @param driver the JDBC driver or the Context class
     * @param url the database URL
     * @param user the user name
     * @param password the password
     * @return the connection
     * @throws RepositoryException if the driver could not be loaded
     * @throws SQLException if the connection could not be established
     */
    public static Connection getConnection(String driver, String url,
            String user, String password) throws RepositoryException,
            SQLException {
        if (driver != null && driver.length() > 0) {
            try {
                Class d = Class.forName(driver);
                if (javax.naming.Context.class.isAssignableFrom(d)) {
                    // JNDI context
                    Context context = (Context) d.newInstance();
                    DataSource ds = (DataSource) context.lookup(url);
                    if (user == null && password == null) {
                        return ds.getConnection();
                    } else {
                        return ds.getConnection(user, password);
                    }
                } else {
                    try {
                        // Workaround for Apache Derby:
                        // The JDBC specification recommends the Class.forName method without the .newInstance() method call,
                        // but it is required after a Derby 'shutdown'.
                        d.newInstance();
                    } catch (Throwable e) {
                        // Ignore exceptions
                        // There's no requirement that a JDBC driver class has a public default constructor
                    }
                }
            } catch (ClassNotFoundException e) {
                throw new RepositoryException("Could not load class " + driver, e);
            } catch (InstantiationException e) {
                throw new RepositoryException("Could not instantiate context " + driver, e);
            } catch (IllegalAccessException e) {
                throw new RepositoryException("Could not instantiate context " + driver, e);
            } catch (NamingException e) {
                throw new RepositoryException("Naming exception using " + driver + " url: " + url, e);
            }
        }
        return DriverManager.getConnection(url, user, password);
    }
    
    
    public Map<String, Map<String, String>> getConfiguration() {
		
		Map<String, Map<String, String>> config = new HashMap<String, Map<String,String>>();
		Map<String, String> jcr = new HashMap<String, String>();
		
		jcr.put("name", repository.getDescriptor(Repository.REP_NAME_DESC));
		jcr.put("version", repository.getDescriptor(Repository.REP_VERSION_DESC));
		jcr.put("vendor", repository.getDescriptor(Repository.REP_VENDOR_DESC));
		config.put("repository", jcr);
		
		return config;
    }


}
