/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.mom.jcrperf;

import java.util.LinkedHashMap;
import java.util.Map;

import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;

import com.mom.jcrperf.tests.LoginLogoutTest;
import com.mom.jcrperf.tests.BigFileReadTest;
import com.mom.jcrperf.tests.BigFileWriteTest;
import com.mom.jcrperf.tests.ConcurrentReadTest;
import com.mom.jcrperf.tests.ConcurrentReadWriteTest;
import com.mom.jcrperf.tests.CreateManyChildNodesTest;
import com.mom.jcrperf.tests.LoginTest;
import com.mom.jcrperf.tests.PerformanceTest;
import com.mom.jcrperf.tests.RefreshTest;
import com.mom.jcrperf.tests.SimpleSearchTest;
import com.mom.jcrperf.tests.SmallFileReadTest;
import com.mom.jcrperf.tests.SmallFileWriteTest;
import com.mom.jcrperf.tests.TransientManyChildNodesTest;
import com.mom.jcrperf.tests.TransientSomeChildNodesTest;
import com.mom.jcrperf.tests.UpdateManyChildNodesTest;
import com.mom.jcrperf.tests.VersionedBundleTest;
import com.mom.jcrperf.tests.VersionedTreeTest;

public class PerformanceTestSuite {

	private ITestableRepository testable;

	public ITestableRepository getTestable() {
		return testable;
	}

	public void setTestable(ITestableRepository testable) {
		this.testable = testable;
	}

	public int getWarmup() {
		return warmup;
	}

	public void setWarmup(int warmup) {
		this.warmup = warmup;
	}

	public int getWarmuptime() {
		return warmuptime;
	}

	public void setWarmuptime(int warmuptime) {
		this.warmuptime = warmuptime;
	}

	public int getRuntime() {
		return runtime;
	}

	public void setRuntime(int runtime) {
		this.runtime = runtime;
	}

	/**
	 * Number of iterations to run to make sure the code has been 
	 * assembled by the JIT
	 */
	private int warmup = 6;
	/**
	 * Maximum number of seconds the warmup should run for
	 */
	private int warmuptime = 20;
	/**
	 * Maximum number of seconds the actual tests should run for
	 */
	private int runtime = 5;

	public DescriptiveStatistics runTest(PerformanceTest test) throws Exception {
		DescriptiveStatistics statistics = new DescriptiveStatistics();

		test.setRepository(testable.getRepository());
		test.setCredentials(testable.getCredentials());

		test.beforeSuite();

		// Run a few iterations to warm up the system
		// Always run six since this seems to be needed to make the JIT pay attention
		long warmupEnd = System.currentTimeMillis() + warmuptime * 1000;
		int warmups = 0;
		while (warmups < warmup && System.currentTimeMillis() < warmupEnd) {
			test.beforeTest();
			test.runTest();
			test.afterTest();
			warmups++;
		}

		// Run test iterations, and capture the execution times
		long runtimeEnd = System.currentTimeMillis() + runtime * 1000;
		while (System.currentTimeMillis() < runtimeEnd) {
			test.beforeTest();
			long start = System.currentTimeMillis();
			test.runTest();
			statistics.addValue(System.currentTimeMillis() - start);
			test.afterTest();
		}

		test.afterSuite();

		return statistics;
	}

	public void run(PerformanceTest test, boolean outputXml) throws Exception {
		
		DescriptiveStatistics statistics = runTest(test);
		
		if(outputXml) {
			
			System.out.println(
					"<test" +
							" name=\""+test.toString()+"\"" +
							" mean=\""+statistics.getMean()+"\"" +
							" sdev=\""+statistics.getStandardDeviation()+"\"" +
							" min=\""+statistics.getMin()+"\"" +
							" max=\""+statistics.getMax()+"\"" +
							" count=\""+statistics.getN()+"\""+
					" />"
					);
			
		} else {
			System.out.format("%-36.36s  %6.0f  %6.0f  %6.0f  %6.0f  %6d%n", test, statistics.getMean(),
				statistics.getStandardDeviation(), statistics.getMin(), statistics.getMax(), statistics.getN());
		}
	}

    
    public Map<String, PerformanceTest> getTestsList() {
		Map<String, PerformanceTest> menu = new LinkedHashMap<String, PerformanceTest>();
		menu.put("Login", new LoginTest());
		menu.put("LoginLogout", new LoginLogoutTest());
		menu.put("Refresh", new RefreshTest());

		menu.put("ConcurrentRead", new ConcurrentReadTest());
		menu.put("ConcurrentReadWrite", new ConcurrentReadWriteTest());

		menu.put("CreateManyChildNodes", new CreateManyChildNodesTest());
		menu.put("TransientManyChildNodes", new TransientManyChildNodesTest());
		menu.put("TransientSomeChildNodes", new TransientSomeChildNodesTest());
		menu.put("UpdateManyChildNodes", new UpdateManyChildNodesTest());
		menu.put("VersionedTree", new VersionedTreeTest());
		menu.put("VersionedBundle", new VersionedBundleTest());
		
		menu.put("SimpleSearch", new SimpleSearchTest());

		menu.put("SmallFileRead", new SmallFileReadTest());
		menu.put("SmallFileWrite", new SmallFileWriteTest());

		menu.put("BigFileRead", new BigFileReadTest());
		menu.put("BigFileWrite", new BigFileWriteTest());
		return menu;
    }

}
